@@ -2,14 +2,14 @@
 import gtk
 
 from kiwi.datatypes import ValidationError
-from kiwi.ui.widgets.combobox import ComboBoxEntry
-from kiwi.ui.delegates import Delegate, SlaveDelegate
+from kiwi.ui.widgets.combo import ProxyComboEntry
+from kiwi.ui.delegates import GladeDelegate, SlaveDelegate
 
-class Dialog(Delegate):
+class Dialog(GladeDelegate):
     def __init__(self):
-        Delegate.__init__(self, gladefile='lang',
-                          widgets=['ok_button'],
-                          delete_handler=self.quit_if_last)
+        GladeDelegate.__init__(self, gladefile='lang',
+                               widgets=['ok_button'],
+                               delete_handler=self.quit_if_last)
         self.register_validate_function(self.validity)
 
     def validity(self, data):
@@ -28,7 +28,7 @@ class English(SlaveDelegate):
         label.show()
         box.pack_start(label, False, False)
 
-        combo = ComboBoxEntry()
+        combo = ProxyComboEntry()
         combo.set_property('model-attribute', 'number')
         combo.set_property('data-type', 'str')
         combo.prefill(['One', 'Two', 'Three'])
@@ -52,7 +52,7 @@ class Swedish(SlaveDelegate):
         label.show()
         box.pack_start(label, False, False)
 
-        combo = ComboBoxEntry()
+        combo = ProxyComboEntry()
         combo.set_property('model-attribute', 'nummer')
         combo.set_property('data-type', 'str')
         combo.prefill(['Ett', u'Tv', 'Tre'])
