@@ -26,7 +26,11 @@
 import gtk
 from gtk import gdk, keysyms
 
+from kiwi.component import implements
+from kiwi.interfaces import IEasyCombo
+from kiwi.enums import ComboColumn, ComboMode
 from kiwi.ui.entry import KiwiEntry
+from kiwi.ui.entrycompletion import KiwiEntryCompletion
 from kiwi.utils import gsignal, type_register
 
 class _ComboEntryPopup(gtk.Window):
@@ -42,6 +46,8 @@ class _ComboEntryPopup(gtk.Window):
         # default value from other toolkits
         self._visible_rows = 10
         self._initial_text = None
+        self._popping_up = False
+        self._filter_model = None
 
         frame = gtk.Frame()
         frame.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
@@ -80,16 +86,29 @@ class _ComboEntryPopup(gtk.Window):
         self.set_resizable(False)
         self.set_screen(comboentry.get_screen())
 
-    def popup(self, text=None):
+    def popup(self, text=None, filter=False):
         """
         Shows the list of options. And optionally selects an item
         @param text: text to select
+        @param filter: filter the list of options. A filter_model must be
+        set using L{set_model}()
         """
         combo = self._comboentry
         if not (combo.flags() & gtk.REALIZED):
             return
 
         treeview = self._treeview
+
+        if filter and self._filter_model:
+            model = self._filter_model
+        else:
+            model = self._model
+
+        if not len(model):
+            return
+
+        treeview.set_model(model)
+
         toplevel = combo.get_toplevel()
         if isinstance(toplevel, gtk.Window) and toplevel.group:
             toplevel.group.add_window(self)
@@ -105,14 +124,23 @@ class _ComboEntryPopup(gtk.Window):
 
         treeview.set_hover_expand(True)
         selection = treeview.get_selection()
+        selection.unselect_all()
         if text:
-            for row in treeview.get_model():
+            for row in model:
                 if text in row:
                     selection.select_iter(row.iter)
                     treeview.scroll_to_cell(row.path, use_align=True,
                                             row_align=0.5)
                     treeview.set_cursor(row.path)
                     break
+
+        self._popping_up = True
+
+        if filter:
+            # do not grab if its a completion
+            return
+
+        # Grab window
         self.grab_focus()
 
         if not (self._treeview.flags() & gtk.HAS_FOCUS):
@@ -141,6 +169,10 @@ class _ComboEntryPopup(gtk.Window):
         self._label.set_text(text)
 
     def set_model(self, model):
+        if isinstance(model, gtk.TreeModelFilter):
+            self._filter_model = model
+            model = model.get_model()
+
         self._treeview.set_model(model)
         self._model = model
 
@@ -237,7 +269,8 @@ class _ComboEntryPopup(gtk.Window):
             self._sw.set_policy(gtk.POLICY_ALWAYS, vpolicy)
             pwidth, pheight = self.size_request()
 
-        rows = len(self._model)
+        rows = len(self._treeview.get_model())
+
         if rows > self._visible_rows:
             rows = self._visible_rows
             self._sw.set_policy(hpolicy, gtk.POLICY_ALWAYS)
@@ -277,15 +310,33 @@ class _ComboEntryPopup(gtk.Window):
         return x, y, width, height
 
     def get_selected_iter(self):
-        return self._selection.get_selected()[1]
+        model, treeiter = self._selection.get_selected()
+
+        # if the model currently being used is a TreeModelFiter, convert
+        # the iter to be a TreeModel iter (witch is what the user expects)
+        if isinstance(model, gtk.TreeModelFilter) and treeiter:
+            treeiter = model.convert_iter_to_child_iter(treeiter)
+        return treeiter
 
     def set_selected_iter(self, iter):
+        model = self._treeview.get_model()
+
+        # Since the user passed a TreeModel iter, if the model currently
+        # being used is a TreeModelFiter, convert it to be a TreeModelFiter
+        # iter
+        if isinstance(model, gtk.TreeModelFilter):
+            iter = model.convert_child_iter_to_iter(iter)
         self._selection.select_iter(iter)
+
 type_register(_ComboEntryPopup)
 
 class ComboEntry(gtk.HBox):
+
+    implements(IEasyCombo)
+
     gsignal('changed')
     gsignal('activate')
+
     def __init__(self, entry=None):
         """
         @param entry: a gtk.Entry subclass to use
@@ -296,6 +347,7 @@ class ComboEntry(gtk.HBox):
         if not entry:
             entry = KiwiEntry()
 
+        self.mode = ComboMode.UNKNOWN
         self.entry = entry
         self.entry.connect('activate',
                            self._on_entry__activate)
@@ -324,7 +376,9 @@ class ComboEntry(gtk.HBox):
         self._popup.connect('hide', self._on_popup__hide)
         self._popup.set_size_request(-1, 24)
 
-        completion = gtk.EntryCompletion()
+        completion = KiwiEntryCompletion()
+        completion.set_popup_window(self._popup)
+        completion.set_treeview(self._popup._treeview)
         self.entry.set_completion(completion)
         self.set_model(completion.get_model())
 
@@ -338,7 +392,9 @@ class ComboEntry(gtk.HBox):
     def _on_entry_completion__match_selected(self, completion, model, iter):
         # the iter we receive is specific to the tree model filter used
         # In the entry completion, convert it to an iter in the real model
-        self.set_active_iter(model.convert_iter_to_child_iter(iter))
+        if isinstance(model, gtk.TreeModelFilter):
+            iter = model.convert_iter_to_child_iter(iter)
+        self.set_active_iter(iter)
 
     def _on_entry__activate(self, entry):
         self.emit('activate')
@@ -385,7 +441,7 @@ class ComboEntry(gtk.HBox):
         self._popping_down = False
 
     def _on_popup__text_selected(self, popup, text):
-        self.entry.set_text(text)
+        self.set_text(text)
         popup.popdown()
         self.entry.grab_focus()
         self.entry.set_position(len(self.entry.get_text()))
@@ -415,18 +471,16 @@ class ComboEntry(gtk.HBox):
 
     def popup(self):
         """
-        Hide the popup window
+        Show the popup window
         """
         self._popup.popup(self.entry.get_text())
 
     def popdown(self):
         """
-        Show the popup window
+        Hide the popup window
         """
         self._popup.popdown()
 
-    # Entry interface
-
     def set_text(self, text):
         """
         @param text:
@@ -439,8 +493,6 @@ class ComboEntry(gtk.HBox):
         """
         return self.entry.get_text()
 
-    # ComboMixin interface
-
     def set_model(self, model):
         """
         Set the tree model to model
@@ -478,31 +530,52 @@ class ComboEntry(gtk.HBox):
         """
         return self._popup.get_selected_iter()
 
+
+    def get_mode(self):
+        return self.mode
+
+    def set_label_text(self, text):
+        self._popup.set_label_text(text)
+
+    # IEasyCombo interface
+
+    def clear(self):
+        """Removes all items from list"""
+        self._model.clear()
+        self.entry.set_text("")
+
     def prefill(self, itemdata, sort=False):
         """
-        See L{kiwi.ui.widgets.entry}
+        See L{kiwi.interfaces.IEasyCombo.prefill}
         """
         self._model.clear()
         self.entry.prefill(itemdata, sort)
+        self.mode = self.entry.get_mode()
 
     def select_item_by_data(self, data):
         """
-        @param data: object to select
+        See L{kiwi.interfaces.IEasyCombo.select_item_by_data}
         """
         treeiter = self.entry.get_iter_by_data(data)
         self.set_active_iter(treeiter)
 
     def select_item_by_label(self, text):
         """
-        @param text: text to select
+        See L{kiwi.interfaces.IEasyCombo.select_item_by_label}
         """
         treeiter = self.entry.get_iter_by_label(text)
         self.set_active_iter(treeiter)
 
+    def select_item_by_position(self, position):
+        """
+        See L{kiwi.interfaces.IEasyCombo.select_item_by_position}
+        """
+        row = self._model[position]
+        self.set_active_iter(row.iter)
+
     def get_selected(self):
         """
-        @returns: selected text or item or None if nothing
-          is selected
+        See L{kiwi.interfaces.IEasyCombo.get_selected}
         """
         treeiter = self.get_active_iter()
         if treeiter:
@@ -510,21 +583,71 @@ class ComboEntry(gtk.HBox):
 
     def get_selected_label(self):
         """
-        @returns: the label of the currently selected item
+        See L{kiwi.interfaces.IEasyCombo.get_selected_label}
         """
         treeiter = self.get_active_iter()
         if treeiter:
             return self.entry.get_selected_label(treeiter)
 
+    def get_selected_data(self):
+        """
+        See L{kiwi.interfaces.IEasyCombo.get_selected_data}
+        """
+        treeiter = self.get_active_iter()
+        if treeiter:
+            return self.entry.get_selected_data(treeiter)
+
     def select(self, obj):
         """
-        @param obj: data or text to select
+        See L{kiwi.interfaces.IEasyCombo.select}
         """
         treeiter = self.entry.get_iter_from_obj(obj)
         self.set_active_iter(treeiter)
 
-    def set_label_text(self, text):
-        self._popup.set_label_text(text)
+    def append_item(self, label, data=None):
+        """
+        See L{kiwi.interfaces.IEasyCombo.append_item}
+        """
+        if not isinstance(label, basestring):
+            raise TypeError("label must be string, found %s" % label)
+
+        if self.mode == ComboMode.UNKNOWN:
+            if data is not None:
+                self.mode = ComboMode.DATA
+            else:
+                self.mode = ComboMode.STRING
+
+        model = self._model
+        if self.mode == ComboMode.STRING:
+            if data is not None:
+                raise TypeError("data can not be specified in string mode")
+            model.append((label, None))
+        elif self.mode == ComboMode.DATA:
+            if data is None:
+                raise TypeError("data must be specified in string mode")
+            model.append((label, data))
+        else:
+            raise AssertionError
+
+    def get_model_strings(self):
+        """
+        See L{kiwi.interfaces.IEasyCombo.get_model_strings}
+        """
+        return [row[ComboColumn.LABEL] for row in self._model]
+
+    def get_model_items(self):
+        """
+        See L{kiwi.interfaces.IEasyCombo.get_model_items}
+        """
+        if self.mode != ComboMode.DATA:
+            raise TypeError("get_model_items can only be used in data mode")
+
+        model = self._model
+        items = {}
+        for row in model:
+            items[row[ComboColumn.LABEL]] = row[ComboColumn.DATA]
+
+        return items
 
     # IconEntry
 
@@ -534,6 +657,9 @@ class ComboEntry(gtk.HBox):
     def update_background(self, color):
         self.entry.update_background(color)
 
+    def get_background(self):
+        return self.entry.get_background()
+
     def get_icon_window(self):
         return self.entry.get_icon_window()
 
