@@ -97,6 +97,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
         self.connect('insert-text', self._on_insert_text)
         self.connect('delete-text', self._on_delete_text)
+        self.connect_after('grab-focus', self._after_grab_focus)
 
         self._current_object = None
         self._mode = ENTRY_MODE_TEXT
@@ -307,6 +308,15 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         completion = self._get_completion()
         completion.set_match_func(match_func)
 
+    def is_empty(self):
+        text = self.get_text()
+        if self._mask:
+            empty = self.get_empty_mask()
+        else:
+            empty = ''
+
+        return text == empty
+        
     # Private
 
     def _really_delete_text(self, start, end):
@@ -480,7 +490,14 @@ class KiwiEntry(PropertyObject, gtk.Entry):
 
         self.stop_emission('delete-text')
 
+    def _after_grab_focus(self, widget):
+        # The text is selectet in grab-focus, so this needs to be done after
+        # that:
+        if self.is_empty():
+            self.set_position(0)
+
     # IconEntry
+
     def set_tooltip(self, text):
         self._icon.set_tooltip(text)
 
@@ -499,23 +516,10 @@ class KiwiEntry(PropertyObject, gtk.Entry):
     # IComboMixin
 
     def prefill(self, itemdata, sort=False):
-        """Fills the Combo with listitems corresponding to the itemdata
-        provided.
-
-        Parameters:
-          - itemdata is a list of strings or tuples, each item corresponding
-            to a listitem. The simple list format is as follows::
-
-            >>> [ label0, label1, label2 ]
-
-            If you require a data item to be specified for each item, use a
-            2-item tuple for each element. The format is as follows::
-
-            >>> [ ( label0, data0 ), (label1, data1), ... ]
-
-          - Sort is a boolean that specifies if the list is to be sorted by
-            label or not. By default it is not sorted
         """
+        See L{kiwi.interfaces.IEasyCombo.prefill}
+        """
+
         if not isinstance(itemdata, (list, tuple)):
             raise TypeError("'data' parameter must be a list or tuple of item "
                             "descriptions, found %s") % type(itemdata)
@@ -609,6 +613,11 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         model = completion.get_model()
         return model[treeiter][COL_TEXT]
 
+    def get_selected_data(self, treeiter):
+        completion = self._get_completion()
+        model = completion.get_model()
+        return model[treeiter][COL_OBJECT]
+
     def get_iter_from_obj(self, obj):
         mode = self._mode
         if mode == ENTRY_MODE_TEXT:
