@@ -30,14 +30,16 @@ Defines the View classes that are included in the Kiwi Framework, which
 are the base of Delegates and Proxies.
 """
 
+import os
 import re
 import string
 
+
 import gobject
 import gtk
 from gtk import gdk
 
-from kiwi.environ import is_gazpacho_required
+from kiwi.environ import environ
 from kiwi.interfaces import IValidatableProxyWidget
 from kiwi.log import Logger
 from kiwi.utils import gsignal, type_register
@@ -93,7 +95,7 @@ class SignalBroker(object):
         Offers autoconnection of widget signals based on function names.
         You simply need to define your controller method in the format::
 
-            def on_widget_name__signal_name(self, widget, *args):
+            def on_widget_name__signal_name(self, widget):
 
         In other words, start the method by "on_", followed by the
         widget name, followed by two underscores ("__"), followed by the
@@ -105,7 +107,6 @@ class SignalBroker(object):
 
         for fname in methods.keys():
             # `on_x__y' has 7 chars and is the smallest possible handler
-            # (though illegal, of course, since the signal name x is bogus)
             if len(fname) < 7:
                 continue
             match = method_regex.match(fname)
@@ -128,9 +129,8 @@ class SignalBroker(object):
                 else:
                     signal_id = widget.connect(signal, methods[fname])
             except TypeError:
-                log.warn("Widget %s doesn't provide a signal %s" % (
-                widget.__class__, signal))
-                continue
+                raise TypeError("Widget %s doesn't provide a signal %s" % (
+                                widget.__class__, signal))
             self._autoconnected.setdefault(widget, []).append((
                 signal, signal_id))
 
@@ -197,7 +197,6 @@ class SlaveView(gobject.GObject):
     widgets = []
     toplevel_name = None
     gladefile = None
-    gladename = None
     domain = None
 
     # This signal is emited when the view wants to return a result value
@@ -208,7 +207,7 @@ class SlaveView(gobject.GObject):
     gsignal('validation-changed', bool)
 
     def __init__(self, toplevel=None, widgets=None, gladefile=None,
-                 gladename=None, toplevel_name=None, domain=None):
+                 toplevel_name=None, domain=None):
         """ Creates a new SlaveView. Sets up self.toplevel and self.widgets
         and checks for reserved names.
         """
@@ -232,11 +231,9 @@ class SlaveView(gobject.GObject):
         self.toplevel = toplevel or klass.toplevel
         self.widgets = widgets or klass.widgets
         self.gladefile = gladefile or klass.gladefile
-        self.gladename = gladename or klass.gladename
         self.toplevel_name = (toplevel_name or
                               klass.toplevel_name or
-                              self.gladefile or
-                              self.gladename)
+                              self.gladefile)
         self.domain = domain or klass.domain
 
         self._check_reserved()
@@ -261,7 +258,7 @@ class SlaveView(gobject.GObject):
 
     def _check_reserved(self):
         for reserved in ["widgets", "toplevel", "gladefile",
-                         "gladename", "tree", "model", "controller"]:
+                         "tree", "model", "controller"]:
             # XXX: take into account widget constructor?
             if reserved in self.widgets:
                 raise AttributeError(
@@ -271,10 +268,7 @@ class SlaveView(gobject.GObject):
     def _get_toplevel(self):
         toplevel = self.toplevel
         if not toplevel and self.toplevel_name:
-            if self._glade_adaptor:
-                toplevel = self._glade_adaptor.get_widget(self.toplevel_name)
-            else:
-                toplevel = getattr(self, self.toplevel_name, None)
+            toplevel = self.get_widget(self.toplevel_name)
 
         if not toplevel:
             raise TypeError("A View requires an instance variable "
@@ -293,9 +287,7 @@ class SlaveView(gobject.GObject):
         if not self.gladefile:
             return
 
-        glade_adaptor = _open_glade(self, self.gladefile,
-                                    self.widgets, self.gladename,
-                                    self.domain)
+        glade_adaptor = _open_glade(self, self.gladefile, self.domain)
 
         container_name = self.toplevel_name
         if not container_name:
@@ -347,7 +339,12 @@ class SlaveView(gobject.GObject):
     def get_widget(self, name):
         """Retrieves the named widget from the View"""
         name = string.replace(name, '.', '_')
-        widget = getattr(self, name, None)
+
+        if self._glade_adaptor:
+            widget = self._glade_adaptor.get_widget(name)
+        else:
+            widget = getattr(self, name, None)
+
         if widget is None:
             raise AttributeError("Widget %s not found in view %s"
                                  % (name, self))
@@ -533,12 +530,7 @@ class SlaveView(gobject.GObject):
         else: # slaveview
             new_widget = shell
 
-        # if our widgets are in a glade file get the placeholder from them
-        # or take it from the view itself otherwise
-        if self._glade_adaptor:
-            placeholder = self._glade_adaptor.get_widget(name)
-        else:
-            placeholder = getattr(self, name, None)
+        placeholder = self.get_widget(name)
 
         if not placeholder:
             raise AttributeError(
@@ -566,10 +558,8 @@ class SlaveView(gobject.GObject):
         # Merge the sizegroups of the slave that is being attached  with the
         # sizegroups of where it is being attached to. Only the sizegroups
         # with the same name will be merged.
-        if slave._glade_adaptor:
-            sizegroups = slave._glade_adaptor.get_sizegroups()
-            for sizegroup in sizegroups:
-                self._merge_sizegroup(sizegroup)
+        for sizegroup in slave.get_sizegroups():
+            self._merge_sizegroup(sizegroup)
 
         if isinstance(placeholder, gtk.EventBox):
             # standard mechanism
@@ -592,9 +582,9 @@ class SlaveView(gobject.GObject):
         # call slave's callback
         slave.on_attach(self)
 
-        slave.connect_object('validation-changed',
-                             self._on_child__validation_changed,
-                             name)
+        slave.connect('validation-changed',
+                      self._on_child__validation_changed,
+                      name)
 
         for notebook in self._notebooks:
             for child in notebook.get_children():
@@ -613,6 +603,15 @@ class SlaveView(gobject.GObject):
         # return placeholder we just removed
         return placeholder
 
+    def get_sizegroups(self):
+        """
+        @returns: a list of sizegroups for the current view.
+        """
+        if not self._glade_adaptor:
+            return []
+
+        return self._glade_adaptor.get_sizegroups()
+
     def _merge_sizegroup(self, other_sizegroup):
         # Merge sizegroup from other with self that have the same name.
         # Actually, no merging is being done, since the old group is preserved
@@ -739,9 +738,9 @@ class SlaveView(gobject.GObject):
                 continue
 
             try:
-                widget.connect_object('validation-changed',
-                                      self._on_child__validation_changed,
-                                      widget_name)
+                widget.connect('validation-changed',
+                               self._on_child__validation_changed,
+                               widget_name)
             except TypeError:
                 raise AssertionError("%r does not have a validation-changed "
                                      "signal." % widget)
@@ -754,10 +753,16 @@ class SlaveView(gobject.GObject):
     # Validation
     #
 
-    def _on_child__validation_changed(self, name, value):
+    def _on_child__validation_changed(self, child, value, name):
         # Children of the view, eg slaves or widgets are connected to
         # this signal. When validation changes of a validatable child
         # this callback is called
+        if isinstance(child, gtk.Widget):
+            # Force invisible and insensitive widgets to be valid
+            if (not child.get_property('visible') or
+                not child.get_property('sensitive')):
+                value = True
+
         self._validation[name] = value
 
         self.check_and_notify_validity()
@@ -822,10 +827,9 @@ class BaseView(SlaveView):
     """A view with a toplevel window."""
 
     def __init__(self, toplevel=None, widgets=None, gladefile=None,
-                 gladename=None, toplevel_name=None, domain=None,
-                 delete_handler=None):
-        SlaveView.__init__(self, toplevel, widgets, gladefile, gladename,
-                           toplevel_name, domain)
+                 toplevel_name=None, domain=None, delete_handler=None):
+        SlaveView.__init__(self, toplevel, widgets, gladefile, toplevel_name,
+                           domain)
 
         if not isinstance(self.toplevel, gtk.Window):
             raise TypeError("toplevel widget must be a Window "
@@ -843,8 +847,7 @@ class BaseView(SlaveView):
         if not self.gladefile:
             return
 
-        return _open_glade(self, self.gladefile, self.widgets,
-                           self.gladename, self.domain)
+        return _open_glade(self, self.gladefile, self.domain)
 
     #
     # Hook for keypress handling
@@ -942,26 +945,58 @@ class BaseView(SlaveView):
         self.toplevel.hide()
         self.quit_if_last()
 
-WidgetTree = None
+def _get_gazpacho():
+    try:
+        from kiwi.ui.gazpacholoader import GazpachoWidgetTree
+    except ImportError:
+        return
+    return GazpachoWidgetTree
 
-def _open_glade(view, gladefile, widgets, name, domain):
-    global WidgetTree
-    if not WidgetTree:
-        try:
-            from gazpacho.loader import loader
-            loader # pyflakes
-        except ImportError, e:
-            if is_gazpacho_required():
-                raise RuntimeError(
-                    "Gazpacho is required, but could not be found: %s" % e)
-            else:
-                try:
-                    from kiwi.ui.libgladeloader import LibgladeWidgetTree as WT
-                    WidgetTree = WT
-                except ImportError:
-                    raise RuntimeError("Could not find a glade parser library")
-        else:
-            from kiwi.ui.gazpacholoader import GazpachoWidgetTree as WT
-            WidgetTree = WT
+def _get_libglade():
+    try:
+        from kiwi.ui.libgladeloader import LibgladeWidgetTree
+    except ImportError:
+        return
+    return LibgladeWidgetTree
 
-    return WidgetTree(view, gladefile, widgets, name, domain)
+def _open_glade(view, gladefile, domain):
+    if not gladefile:
+        raise ValueError("A gladefile wasn't provided.")
+    elif not isinstance(gladefile, basestring):
+        raise TypeError(
+              "gladefile should be a string, found %s" % type(gladefile))
+
+    filename = os.path.splitext(os.path.basename(gladefile))[0]
+    gladefile = environ.find_resource("glade", filename + '.glade')
+
+    fp = open(gladefile)
+    sniff = fp.read(200)
+    fp.close()
+
+    # glade-2
+    #<?xml version="1.0" standalone="no"?> <!--*- mode: xml -*-->
+    #<!DOCTYPE glade-interface SYSTEM "http://glade.gnome.org/glade-2.0.dtd">
+
+    # glade-3
+    # <?xml version="1.0" encoding="UTF-8" standalone="no"?>
+    # <!DOCTYPE glade-interface SYSTEM "glade-2.0.dtd">
+    if 'glade-2.0.dtd' in sniff:
+        WidgetTree = _get_libglade()
+        loader_name = 'libglade'
+    else:
+        # gazpacho:
+        #<?xml version="1.0" standalone="no"?> <!--*- mode: xml -*-->
+        #<!DOCTYPE glade-interface SYSTEM "http://gazpacho.sicem.biz/gazpacho-0.1.dtd">
+        if not 'gazpacho-0.1.dtd' in sniff:
+            log.warning("Could not determine type/dtd of gladefile %s" % gladefile)
+
+        WidgetTree = _get_gazpacho()
+        loader_name = 'gazpacho.loader'
+
+    # None means, failed to import
+    if WidgetTree is None:
+        raise RuntimeError(
+            "Could not find %s, it needs to be installed to "
+            "load the gladefile %s" % (loader_name, gladefile))
+
+    return WidgetTree(view, gladefile, domain)
