@@ -1,15 +1,19 @@
 #!/usr/bin/env python
-import utils
-
+# -*- coding: utf-8 -*-
 import unittest
 
+import gobject
+
 from kiwi import ValueUnset
 from kiwi import datatypes
-from kiwi.ui.widgets.entry import Entry
+from kiwi.ui.entry import KiwiEntry
+from kiwi.ui.widgets.entry import ProxyEntry
+
+import utils
 
 class EntryTest(unittest.TestCase):
     def testModel(self):
-        entry = Entry()
+        entry = ProxyEntry()
         entry.set_text('value')
         entry.set_property("data-type", "str")
         self.assertEqual(entry.read(), 'value')
@@ -17,7 +21,7 @@ class EntryTest(unittest.TestCase):
     # FIXME
     def _testValidDataType(self):
 
-        entry = Entry()
+        entry = ProxyEntry()
         entry.set_property("data-type", "date")
         # let's make the entry complain!
         entry.set_text("string")
@@ -44,9 +48,9 @@ class EntryTest(unittest.TestCase):
             entry.set_text("23.400.000,2")
             self.assertEqual(entry.read(), ValueUnset)
 
-    def testMask(self):
-        e = Entry()
-        e.set_mask('%3d.%3d')
+    def testDigitMask(self):
+        e = ProxyEntry()
+        e.set_mask('000.000')
         self.assertEqual(e.get_text(), '   .   ')
         self.assertEqual(e.get_field_text(), [None, None])
         e.set_text('123.456')
@@ -56,14 +60,39 @@ class EntryTest(unittest.TestCase):
         self.assertEqual(e.get_text(), '  3.456')
         self.assertEqual(e.get_field_text(), [3, 456])
 
+    def testAsciiMask(self):
+        e = ProxyEntry()
+        e.set_mask('LLLL-L')
+        self.assertEqual(e.get_text(), '    - ')
+        self.assertEqual(e.get_field_text(), ['', ''])
+        e.set_text('abcd-e')
+        self.assertEqual(e.get_text(), 'abcd-e')
+        self.assertEqual(e.get_field_text(), ['abcd', 'e'])
+
+    def testAlphaNumericMask(self):
+        e = ProxyEntry()
+        e.set_mask('&&&-aaa')
+        self.assertEqual(e.get_text(), '   -   ')
+        self.assertEqual(e.get_field_text(), ['', ''])
+        e.set_text('aáé-á1e')
+        self.assertEqual(e.get_text(), 'aáé-á1e')
+        self.assertEqual(e.get_field_text(), ['aáé', 'á1e'])
+
     def testMaskSmallFields(self):
-        e = Entry()
-        e.set_mask('%d.%d.%d')
+        e = ProxyEntry()
+        e.set_mask('0.0.0')
         self.assertEqual(e.get_text(), ' . . ')
         self.assertEqual(e.get_field_text(), [None, None, None])
         e.set_text('1.2.3')
         self.assertEqual(e.get_text(), '1.2.3')
         self.assertEqual(e.get_field_text(), [1, 2, 3])
 
+    def testGType(self):
+        entry = KiwiEntry()
+        self.assertEqual(gobject.type_name(entry), 'KiwiEntry')
+
+        entry = ProxyEntry()
+        self.assertEqual(gobject.type_name(entry), 'ProxyEntry')
+
 if __name__ == '__main__':
     unittest.main()
